@@ -1,6 +1,6 @@
 package Monitoring::GLPlugin;
 
-=head1 Monitoring::GLPlugin 
+=head1 Monitoring::GLPlugin
 
 Monitoring::GLPlugin - infrastructure functions to build a monitoring plugin
 
@@ -11,8 +11,9 @@ use IO::File;
 use File::Basename;
 use Digest::MD5 qw(md5_hex);
 use Errno;
+use Data::Dumper;
 our $AUTOLOAD;
-*VERSION = \'1.3';
+*VERSION = \'2.0.10';
 
 use constant { OK => 0, WARNING => 1, CRITICAL => 2, UNKNOWN => 3 };
 
@@ -28,8 +29,7 @@ use constant { OK => 0, WARNING => 1, CRITICAL => 2, UNKNOWN => 3 };
 }
 
 sub new {
-  my $class = shift;
-  my %params = @_;
+  my ($class, %params) = @_;
   my $self = {};
   bless $self, $class;
   require Monitoring::GLPlugin::Commandline
@@ -43,7 +43,7 @@ sub new {
 }
 
 sub init {
-  my $self = shift;
+  my ($self) = @_;
   if ($self->opts->can("blacklist") && $self->opts->blacklist &&
       -f $self->opts->blacklist) {
     $self->opts->blacklist = do {
@@ -52,8 +52,7 @@ sub init {
 }
 
 sub dumper {
-  my $self = shift;
-  my $object = shift;
+  my ($self, $object) = @_;
   my $run = $object->{runtime};
   delete $object->{runtime};
   printf STDERR "%s\n", Data::Dumper::Dumper($object);
@@ -61,7 +60,7 @@ sub dumper {
 }
 
 sub no_such_mode {
-  my $self = shift;
+  my ($self) = @_;
   printf "Mode %s is not implemented for this type of device\n",
       $self->opts->mode;
   exit 3;
@@ -71,7 +70,7 @@ sub no_such_mode {
 # framework-related. setup, options
 #
 sub add_default_args {
-  my $self = shift;
+  my ($self) = @_;
   $self->add_arg(
       spec => 'mode=s',
       help => "--mode
@@ -239,8 +238,7 @@ sub add_default_args {
 }
 
 sub add_modes {
-  my $self = shift;
-  my $modes = shift;
+  my ($self, $modes) = @_;
   my $modestring = "";
   my @modes = @{$modes};
   my $longest = length ((reverse sort {length $a <=> length $b} map { $_->[1] } @modes)[0]);
@@ -255,8 +253,7 @@ sub add_modes {
 }
 
 sub add_arg {
-  my $self = shift;
-  my %args = @_;
+  my ($self, %args) = @_;
   if ($args{help} =~ /^--mode/) {
     $args{help} .= "\n".$Monitoring::GLPlugin::plugin->{modestring};
   }
@@ -264,13 +261,12 @@ sub add_arg {
 }
 
 sub mod_arg {
-  my $self = shift;
-  $Monitoring::GLPlugin::plugin->{opts}->mod_arg(@_);
+  my ($self, @arg) = @_;
+  $Monitoring::GLPlugin::plugin->{opts}->mod_arg(@arg);
 }
 
 sub add_mode {
-  my $self = shift;
-  my %args = @_;
+  my ($self, %args) = @_;
   push(@{$Monitoring::GLPlugin::plugin->{modes}}, \%args);
   my $longest = length ((reverse sort {length $a <=> length $b} map { $_->{spec} } @{$Monitoring::GLPlugin::plugin->{modes}})[0]);
   my $format = "       %-".
@@ -284,7 +280,7 @@ sub add_mode {
 }
 
 sub validate_args {
-  my $self = shift;
+  my ($self) = @_;
   if ($self->opts->mode =~ /^my-([^\-.]+)/) {
     my $param = $self->opts->mode;
     $param =~ s/\-/::/g;
@@ -358,17 +354,17 @@ sub validate_args {
       $self->override_opt('statefilesdir', "/var/tmp/".$Monitoring::GLPlugin::plugin->{name});
     }
   }
-  $Monitoring::GLPlugin::plugin->{statefilesdir} = $self->opts->statefilesdir 
+  $Monitoring::GLPlugin::plugin->{statefilesdir} = $self->opts->statefilesdir
       if $self->opts->can("statefilesdir");
   if ($self->opts->can("warningx") && $self->opts->warningx) {
     foreach my $key (keys %{$self->opts->warningx}) {
-      $self->set_thresholds(metric => $key, 
+      $self->set_thresholds(metric => $key,
           warning => $self->opts->warningx->{$key});
     }
   }
   if ($self->opts->can("criticalx") && $self->opts->criticalx) {
     foreach my $key (keys %{$self->opts->criticalx}) {
-      $self->set_thresholds(metric => $key, 
+      $self->set_thresholds(metric => $key,
           critical => $self->opts->criticalx->{$key});
     }
   }
@@ -376,50 +372,57 @@ sub validate_args {
 }
 
 sub set_timeout_alarm {
-  my $self = shift;
-  $SIG{'ALRM'} = sub {
+  my ($self, $timeout) = @_;
+  my $timeout = defined $timeout ? int($timeout) : $self->opts->timeout;
+  my $handler = sub {
     printf "UNKNOWN - %s timed out after %d seconds\n",
         $Monitoring::GLPlugin::plugin->{name}, $self->opts->timeout;
     exit 3;
   };
-  alarm($self->opts->timeout);
+  use POSIX ':signal_h';
+  if ($^O =~ /MSWin/) {
+    local $SIG{'ALRM'} = $handler;
+  } else {
+    my $mask = POSIX::SigSet->new( SIGALRM );
+    my $action = POSIX::SigAction->new(
+        $handler, $mask
+    );   
+    my $oldaction = POSIX::SigAction->new();
+    sigaction(SIGALRM ,$action ,$oldaction );
+  }    
+  alarm($timeout); # 1 second before the global unknown timeout
 }
 
 #########################################################
 # global helpers
 #
 sub set_variable {
-  my $self = shift;
-  my $key = shift;
-  my $value = shift;
+  my ($self, $key, $value) = @_;
   $Monitoring::GLPlugin::variables->{$key} = $value;
 }
 
 sub get_variable {
-  my $self = shift;
-  my $key = shift;
-  my $fallback = shift;
+  my ($self, $key, $fallback) = @_;
   return exists $Monitoring::GLPlugin::variables->{$key} ?
       $Monitoring::GLPlugin::variables->{$key} : $fallback;
 }
 
 sub debug {
-  my $self = shift;
-  my $format = shift;
+  my ($self, $format, @message) = @_;
   my $tracefile = "/tmp/".$Monitoring::GLPlugin::pluginname.".trace";
   $self->{trace} = -f $tracefile ? 1 : 0;
   if ($self->get_variable("verbose") &&
       $self->get_variable("verbose") > $self->get_variable("verbosity", 10)) {
     printf("%s: ", scalar localtime);
-    printf($format, @_);
+    printf($format, @message);
     printf "\n";
   }
   if ($self->{trace}) {
-    my $logfh = new IO::File;
+    my $logfh = IO::File->new();
     $logfh->autoflush(1);
     if ($logfh->open($tracefile, "a")) {
       $logfh->printf("%s: ", scalar localtime);
-      $logfh->printf($format, @_);
+      $logfh->printf($format, @message);
       $logfh->printf("\n");
       $logfh->close();
     }
@@ -427,9 +430,7 @@ sub debug {
 }
 
 sub filter_namex {
-  my $self = shift;
-  my $opt = shift;
-  my $name = shift;
+  my ($self, $opt, $name) = @_;
   if ($opt) {
     if ($self->opts->regexp) {
       if ($name =~ /$opt/i) {
@@ -447,26 +448,22 @@ sub filter_namex {
 }
 
 sub filter_name {
-  my $self = shift;
-  my $name = shift;
+  my ($self, $name) = @_;
   return $self->filter_namex($self->opts->name, $name);
 }
 
 sub filter_name2 {
-  my $self = shift;
-  my $name = shift;
+  my ($self, $name) = @_;
   return $self->filter_namex($self->opts->name2, $name);
 }
 
 sub filter_name3 {
-  my $self = shift;
-  my $name = shift;
+  my ($self, $name) = @_;
   return $self->filter_namex($self->opts->name3, $name);
 }
 
 sub version_is_minimum {
-  my $self = shift;
-  my $version = shift;
+  my ($self, $version) = @_;
   my $installed_version;
   my $newer = 1;
   if ($self->get_variable("version")) {
@@ -496,8 +493,7 @@ sub version_is_minimum {
 }
 
 sub accentfree {
-  my $self = shift;
-  my $text = shift;
+  my ($self, $text) = @_;
   # thanks mycoyne who posted this accent-remove-algorithm
   # http://www.experts-exchange.com/Programming/Languages/Scripting/Perl/Q_23275533.html#a21234612
   my @transformed;
@@ -524,7 +520,7 @@ sub accentfree {
 }
 
 sub dump {
-  my $self = shift;
+  my ($self) = @_;
   my $class = ref($self);
   $class =~ s/^.*:://;
   if (exists $self->{flat_indices}) {
@@ -532,14 +528,14 @@ sub dump {
   } else {
     printf "[%s]\n", uc $class;
   }
-  foreach (grep !/^(info|trace|warning|critical|blacklisted|extendedinfo|flat_indices|indices)/, sort keys %{$self}) {
+  foreach (grep !/^(info|trace|warning|critical|blacklisted|extendedinfo|flat_indices|indices)$/, sort keys %{$self}) {
     printf "%s: %s\n", $_, $self->{$_} if defined $self->{$_} && ref($self->{$_}) ne "ARRAY";
   }
   if ($self->{info}) {
     printf "info: %s\n", $self->{info};
   }
   printf "\n";
-  foreach (grep !/^(info|trace|warning|critical|blacklisted|extendedinfo|flat_indices|indices)/, sort keys %{$self}) {
+  foreach (grep !/^(info|trace|warning|critical|blacklisted|extendedinfo|flat_indices|indices)$/, sort keys %{$self}) {
     if (defined $self->{$_} && ref($self->{$_}) eq "ARRAY") {
       my $have_flat_indices = 1;
       foreach my $obj (@{$self->{$_}}) {
@@ -562,9 +558,7 @@ sub dump {
 }
 
 sub table_ascii {
-  my $self = shift;
-  my $table = shift;
-  my $titles = shift;
+  my ($self, $table, $titles) = @_;
   my $text = "";
   my $column_length = {};
   my $column = 0;
@@ -602,9 +596,7 @@ sub table_ascii {
 }
 
 sub table_html {
-  my $self = shift;
-  my $table = shift;
-  my $titles = shift;
+  my ($self, $table, $titles) = @_;
   my $text = "";
   $text .= "<table style=\"border-collapse:collapse; border: 1px solid black;\">";
   $text .= "<tr>";
@@ -634,7 +626,7 @@ sub table_html {
 }
 
 sub load_my_extension {
-  my $self = shift;
+  my ($self) = @_;
   if ($self->opts->mode =~ /^my-([^-.]+)/) {
     my $class = $1;
     my $loaderror = undef;
@@ -664,7 +656,9 @@ sub load_my_extension {
     }
     my $original_class = ref($self);
     my $original_init = $self->can("init");
+    $self->compatibility_class() if $self->can('compatibility_class');
     bless $self, "My$class";
+    $self->compatibility_methods() if $self->can('compatibility_methods');
     if ($self->isa("Monitoring::GLPlugin")) {
       my $new_init = $self->can("init");
       if ($new_init == $original_init) {
@@ -686,8 +680,7 @@ sub load_my_extension {
 }
 
 sub decode_password {
-  my $self = shift;
-  my $password = shift;
+  my ($self, $password) = @_;
   if ($password && $password =~ /^rfc3986:\/\/(.*)/) {
     $password = $1;
     $password =~ s/%([A-Za-z0-9]{2})/chr(hex($1))/seg;
@@ -696,8 +689,7 @@ sub decode_password {
 }
 
 sub number_of_bits {
-  my $self = shift;
-  my $unit = shift;
+  my ($self, $unit) = @_;
   # https://en.wikipedia.org/wiki/Data_rate_units
   my $bits = {
     'bit' => 1,			# Bit per second
@@ -723,9 +715,11 @@ sub number_of_bits {
     'TB' => 8000000000000,	# Terabyte per second, 1,000 gigabytes per second
     # eigene kreationen
     'Bits' => 1,
-    'KBi' => 1024,
-    'MBi' => 1024 * 1024,
-    'GBi' => 1024 * 1024 * 1024,
+    'Bit' => 1,			# Bit per second
+    'KB' => 1024,		# Kilobyte (like disk kilobyte)
+    'KBi' => 1024,		# -"-
+    'MBi' => 1024 * 1024,	# Megabyte (like disk megabyte)
+    'GBi' => 1024 * 1024 * 1024, # Gigybate (like disk gigybyte)
   };
   if (exists $bits->{$unit}) {
     return $bits->{$unit};
@@ -739,23 +733,23 @@ sub number_of_bits {
 # runtime methods
 #
 sub mode : lvalue {
-  my $self = shift;
+  my ($self) = @_;
   $Monitoring::GLPlugin::mode;
 }
 
 sub statefilesdir {
-  my $self = shift;
+  my ($self) = @_;
   return $Monitoring::GLPlugin::plugin->{statefilesdir};
 }
 
 sub opts { # die beiden _nicht_ in AUTOLOAD schieben, das kracht!
-  my $self = shift;
+  my ($self) = @_;
   return $Monitoring::GLPlugin::plugin->opts();
 }
 
 sub getopts {
-  my $self = shift;
-  my $envparams = shift || [];
+  my ($self, $envparams) = @_;
+  $envparams ||= [];
   $Monitoring::GLPlugin::plugin->getopts();
   # es kann sein, dass beim aufraeumen zum schluss als erstes objekt
   # das $Monitoring::GLPlugin::plugin geloescht wird. in anderen destruktoren
@@ -779,33 +773,32 @@ sub getopts {
 }
 
 sub add_ok {
-  my $self = shift;
-  my $message = shift || $self->{info};
+  my ($self, $message) = @_;
+  $message ||= $self->{info};
   $self->add_message(OK, $message);
 }
 
 sub add_warning {
-  my $self = shift;
-  my $message = shift || $self->{info};
+  my ($self, $message) = @_;
+  $message ||= $self->{info};
   $self->add_message(WARNING, $message);
 }
 
 sub add_critical {
-  my $self = shift;
-  my $message = shift || $self->{info};
+  my ($self, $message) = @_;
+  $message ||= $self->{info};
   $self->add_message(CRITICAL, $message);
 }
 
 sub add_unknown {
-  my $self = shift;
-  my $message = shift || $self->{info};
+  my ($self, $message) = @_;
+  $message ||= $self->{info};
   $self->add_message(UNKNOWN, $message);
 }
 
 sub add_message {
-  my $self = shift;
-  my $level = shift;
-  my $message = shift || $self->{info};
+  my ($self, $level, $message) = @_;
+  $message ||= $self->{info};
   $Monitoring::GLPlugin::plugin->add_message($level, $message)
       unless $self->is_blacklisted();
   if (exists $self->{failed}) {
@@ -818,27 +811,27 @@ sub add_message {
 }
 
 sub clear_ok {
-  my $self = shift;
+  my ($self) = @_;
   $self->clear_messages(OK);
 }
 
 sub clear_warning {
-  my $self = shift;
+  my ($self) = @_;
   $self->clear_messages(WARNING);
 }
 
 sub clear_critical {
-  my $self = shift;
+  my ($self) = @_;
   $self->clear_messages(CRITICAL);
 }
 
 sub clear_unknown {
-  my $self = shift;
+  my ($self) = @_;
   $self->clear_messages(UNKNOWN);
 }
 
 sub clear_all { # deprecated, use clear_messages
-  my $self = shift;
+  my ($self) = @_;
   $self->clear_ok();
   $self->clear_warning();
   $self->clear_critical();
@@ -846,8 +839,7 @@ sub clear_all { # deprecated, use clear_messages
 }
 
 sub set_level {
-  my $self = shift;
-  my $code = shift;
+  my ($self, $code) = @_;
   $code = (qw(ok warning critical unknown))[$code] if $code =~ /^\d+$/;
   $code = lc $code;
   if (! exists $self->{tmp_level}) {
@@ -862,7 +854,7 @@ sub set_level {
 }
 
 sub get_level {
-  my $self = shift;
+  my ($self) = @_;
   return OK if ! exists $self->{tmp_level};
   my $code = OK;
   $code ||= CRITICAL if $self->{tmp_level}->{critical};
@@ -875,19 +867,18 @@ sub get_level {
 # blacklisting
 #
 sub blacklist {
-  my $self = shift;
+  my ($self) = @_;
   $self->{blacklisted} = 1;
 }
 
 sub add_blacklist {
-  my $self = shift;
-  my $list = shift;
+  my ($self, $list) = @_;
   $Monitoring::GLPlugin::blacklist = join('/',
       (split('/', $self->opts->blacklist), $list));
 }
 
 sub is_blacklisted {
-  my $self = shift;
+  my ($self) = @_;
   if (! $self->opts->can("blacklist")) {
     return 0;
   }
@@ -921,7 +912,7 @@ sub is_blacklisted {
         }
       }
     }
-  } 
+  }
   return $self->{blacklisted};
 }
 
@@ -929,16 +920,14 @@ sub is_blacklisted {
 # additional info
 #
 sub add_info {
-  my $self = shift;
-  my $info = shift;
+  my ($self, $info) = @_;
   $info = $self->is_blacklisted() ? $info.' (blacklisted)' : $info;
   $self->{info} = $info;
   push(@{$Monitoring::GLPlugin::info}, $info);
 }
 
 sub annotate_info {
-  my $self = shift;
-  my $annotation = shift;
+  my ($self, $annotation) = @_;
   my $lastinfo = pop(@{$Monitoring::GLPlugin::info});
   $lastinfo .= sprintf ' (%s)', $annotation;
   $self->{info} = $lastinfo;
@@ -946,38 +935,36 @@ sub annotate_info {
 }
 
 sub add_extendedinfo {  # deprecated
-  my $self = shift;
-  my $info = shift;
+  my ($self, $info) = @_;
   $self->{extendedinfo} = $info;
   return if ! $self->opts->extendedinfo;
   push(@{$Monitoring::GLPlugin::extendedinfo}, $info);
 }
 
 sub get_info {
-  my $self = shift;
-  my $separator = shift || ' ';
+  my ($self, $separator) = @_;
+  $separator ||= ' ';
   return join($separator , @{$Monitoring::GLPlugin::info});
 }
 
 sub get_last_info {
-  my $self = shift;
+  my ($self) = @_;
   return pop(@{$Monitoring::GLPlugin::info});
 }
 
 sub get_extendedinfo {
-  my $self = shift;
-  my $separator = shift || ' ';
+  my ($self, $separator) = @_;
+  $separator ||= ' ';
   return join($separator, @{$Monitoring::GLPlugin::extendedinfo});
 }
 
 sub add_summary {  # deprecated
-  my $self = shift;
-  my $summary = shift;
+  my ($self, $summary) = @_;
   push(@{$Monitoring::GLPlugin::summary}, $summary);
 }
 
 sub get_summary {
-  my $self = shift;
+  my ($self) = @_;
   return join(', ', @{$Monitoring::GLPlugin::summary});
 }
 
@@ -985,10 +972,8 @@ sub get_summary {
 # persistency
 #
 sub valdiff {
-  my $self = shift;
-  my $pparams = shift;
+  my ($self, $pparams, @keys) = @_;
   my %params = %{$pparams};
-  my @keys = @_;
   my $now = time;
   my $newest_history_set = {};
   $params{freeze} = 0 if ! $params{freeze};
@@ -1004,18 +989,18 @@ sub valdiff {
   }
   # lookback=99999, freeze=0(default)
   #  nimm den letzten lauf und schreib ihn nach {cold}
-  #  vergleich dann 
+  #  vergleich dann
   #    wenn es frozen gibt, vergleich frozen und den letzten lauf
   #    sonst den letzten lauf und den aktuellen lauf
   # lookback=99999, freeze=1
-  #  wird dann aufgerufen,wenn nach dem freeze=0 ein problem festgestellt wurde 
+  #  wird dann aufgerufen,wenn nach dem freeze=0 ein problem festgestellt wurde
   #     (also als 2.valdiff hinterher)
   #  schreib cold nach frozen
   # lookback=99999, freeze=2
   #  wird dann aufgerufen,wenn nach dem freeze=0 wieder alles ok ist
   #     (also als 2.valdiff hinterher)
   #  loescht frozen
-  #  
+  #
   my $last_values = $self->load_state(%params) || eval {
     my $empty_events = {};
     foreach (@keys) {
@@ -1065,7 +1050,7 @@ sub valdiff {
         } else {
           $last_values->{$_} = $last_values->{frozen}->{$_};
         }
-      } 
+      }
     } elsif ($mode eq "lookback") {
       # find a last_value in the history which fits lookback best
       # and overwrite $last_values->{$_} with historic data
@@ -1078,8 +1063,16 @@ sub valdiff {
           if ($date >= ($now - $self->opts->lookback)) {
             $last_values->{$_} = $last_values->{lookback_history}->{$_}->{$date};
             $last_values->{timestamp} = $date;
+            if (ref($last_values->{$_}) eq "ARRAY") {
+              $self->debug(sprintf "oldest value of %s within lookback is size %s (age %d)",
+                  $_, scalar(@{$last_values->{$_}}), time - $date);
+            } else {
+              $self->debug(sprintf "oldest value of %s within lookback is %s (age %d)",
+                  $_, $last_values->{$_}, time - $date);
+            }
             last;
           } else {
+            $self->debug(sprintf "deprecate %s of age %d", $_, time - $date);
             delete $last_values->{lookback_history}->{$_}->{$date};
           }
         }
@@ -1098,6 +1091,8 @@ sub valdiff {
           if ($mode =~ /lookback_freeze/) {
             # hier koennen delta-werte auch negativ sein, wenn z.b. peers verschwinden
             $self->{'delta_'.$_} = $self->{$_} - $last_values->{$_};
+          } elsif (exists $params{lastarray}) {
+            $self->{'delta_'.$_} = $self->{$_} - $last_values->{$_};
           } else {
             # vermutlich db restart und zaehler alle auf null
             $self->{'delta_'.$_} = $self->{$_};
@@ -1141,14 +1136,26 @@ sub valdiff {
       $empty_events->{$_} = $self->{$_};
       if ($mode =~ /lookback_freeze/) {
         if (exists $last_values->{frozen}->{$_}) {
-          $empty_events->{cold}->{$_} = $last_values->{frozen}->{$_};
+          if (ref($last_values->{frozen}->{$_}) eq "ARRAY") {
+            @{$empty_events->{cold}->{$_}} = @{$last_values->{frozen}->{$_}};
+          } else {
+            $empty_events->{cold}->{$_} = $last_values->{frozen}->{$_};
+          }
         } else {
-          $empty_events->{cold}->{$_} = $last_values->{cold}->{$_};
+          if (ref($last_values->{cold}->{$_}) eq "ARRAY") {
+            @{$empty_events->{cold}->{$_}} = @{$last_values->{cold}->{$_}};
+          } else {
+            $empty_events->{cold}->{$_} = $last_values->{cold}->{$_};
+          }
         }
         $empty_events->{cold}->{timestamp} = $last_values->{cold}->{timestamp};
       }
       if ($mode eq "lookback_freeze_shockfrost") {
-        $empty_events->{frozen}->{$_} = $empty_events->{cold}->{$_};
+        if (ref($empty_events->{cold}->{$_}) eq "ARRAY") {
+          @{$empty_events->{frozen}->{$_}} = @{$empty_events->{cold}->{$_}};
+        } else {
+          $empty_events->{frozen}->{$_} = $empty_events->{cold}->{$_};
+        }
         $empty_events->{frozen}->{timestamp} = $now;
       }
     }
@@ -1156,7 +1163,11 @@ sub valdiff {
     if ($mode eq "lookback") {
       $empty_events->{lookback_history} = $last_values->{lookback_history};
       foreach (@keys) {
-        $empty_events->{lookback_history}->{$_}->{$now} = $self->{$_};
+        if (ref($self->{$_}) eq "ARRAY") {
+          @{$empty_events->{lookback_history}->{$_}->{$now}} = @{$self->{$_}};
+        } else {
+          $empty_events->{lookback_history}->{$_}->{$now} = $self->{$_};
+        }
       }
     }
     if ($mode eq "lookback_freeze_defrost") {
@@ -1168,7 +1179,7 @@ sub valdiff {
 }
 
 sub create_statefilesdir {
-  my $self = shift;
+  my ($self) = @_;
   if (! -d $self->statefilesdir()) {
     eval {
       use File::Path;
@@ -1185,8 +1196,7 @@ sub create_statefilesdir {
 }
 
 sub create_statefile {
-  my $self = shift;
-  my %params = @_;
+  my ($self, %params) = @_;
   my $extension = "";
   $extension .= $params{name} ? '_'.$params{name} : '';
   $extension =~ s/\//_/g;
@@ -1199,16 +1209,13 @@ sub create_statefile {
 }
 
 sub schimpf {
-  my $self = shift;
+  my ($self) = @_;
   printf "statefilesdir %s is not writable.\nYou didn't run this plugin as root, didn't you?\n", $self->statefilesdir();
 }
 
 # $self->protect_value('1.1-flat_index', 'cpu_busy', 'percent');
 sub protect_value {
-  my $self = shift;
-  my $ident = shift;
-  my $key = shift;
-  my $validfunc = shift;
+  my ($self, $ident, $key, $validfunc) = @_;
   if (ref($validfunc) ne "CODE" && $validfunc eq "percent") {
     $validfunc = sub {
       my $value = shift;
@@ -1245,15 +1252,14 @@ sub protect_value {
 }
 
 sub save_state {
-  my $self = shift;
-  my %params = @_;
+  my ($self, %params) = @_;
   $self->create_statefilesdir();
   my $statefile = $self->create_statefile(%params);
   my $tmpfile = $self->statefilesdir().'/check__health_tmp_'.$$;
   if ((ref($params{save}) eq "HASH") && exists $params{save}->{timestamp}) {
     $params{save}->{localtime} = scalar localtime $params{save}->{timestamp};
   }
-  my $seekfh = new IO::File;
+  my $seekfh = IO::File->new();
   if ($seekfh->open($tmpfile, "w")) {
     $seekfh->printf("%s", Data::Dumper::Dumper($params{save}));
     $seekfh->flush();
@@ -1268,18 +1274,18 @@ sub save_state {
 }
 
 sub load_state {
-  my $self = shift;
-  my %params = @_;
+  my ($self, %params) = @_;
   my $statefile = $self->create_statefile(%params);
   if ( -f $statefile) {
     our $VAR1;
     eval {
+      delete $INC{$statefile} if exists $INC{$statefile}; # else unit tests fail
       require $statefile;
     };
     if($@) {
       printf "rumms\n";
     }
-    $self->debug(sprintf "load %s", Data::Dumper::Dumper($VAR1));
+    $self->debug(sprintf "load %s from %s", Data::Dumper::Dumper($VAR1), $statefile);
     return $VAR1;
   } else {
     return undef;
@@ -1290,8 +1296,8 @@ sub load_state {
 # daemon mode
 #
 sub check_pidfile {
-  my $self = shift;
-  my $fh = new IO::File;
+  my ($self) = @_;
+  my $fh = IO::File->new();
   if ($fh->open($self->{pidfile}, "r")) {
     my $pid = $fh->getline();
     $fh->close();
@@ -1319,7 +1325,7 @@ sub check_pidfile {
 }
 
 sub write_pidfile {
-  my $self = shift;
+  my ($self) = @_;
   if (! -d dirname($self->{pidfile})) {
     eval "require File::Path;";
     if (defined(&File::Path::mkpath)) {
@@ -1334,7 +1340,7 @@ sub write_pidfile {
       } split(/\//, dirname($self->{pidfile}));
     }
   }
-  my $fh = new IO::File;
+  my $fh = IO::File->new();
   $fh->autoflush(1);
   if ($fh->open($self->{pidfile}, "w")) {
     $fh->printf("%s", $$);
@@ -1345,8 +1351,124 @@ sub write_pidfile {
   }
 }
 
+sub system_vartmpdir {
+  my ($self) = @_;
+  if ($^O =~ /MSWin/) {
+    return $self->system_tmpdir();
+  } else {
+    return "/var/tmp/".$Monitoring::GLPlugin::pluginname;
+  }
+}
+
+sub system_tmpdir {
+  my ($self) = @_;
+  if ($^O =~ /MSWin/) {
+    return $ENV{TEMP} if defined $ENV{TEMP};
+    return $ENV{TMP} if defined $ENV{TMP};
+    return File::Spec->catfile($ENV{windir}, 'Temp')
+        if defined $ENV{windir};
+    return 'C:\Temp';
+  } else {
+    return "/tmp";
+  }
+}
+
+sub convert_scientific_numbers {
+  my ($self, $n) = @_;
+  # mostly used to convert numbers in scientific notation
+  if ($n =~ /^\s*\d+\s*$/) {
+    return $n;
+  } elsif ($n =~ /^\s*([-+]?)(\d*[\.,]*\d*)[eE]{1}([-+]?)(\d+)\s*$/) {
+    my ($vor, $num, $sign, $exp) = ($1, $2, $3, $4);
+    $n =~ s/E/e/g;
+    $n =~ s/,/\./g;
+    $num =~ s/,/\./g;
+    my $sig = $sign eq '-' ? "." . ($exp - 1 + length $num) : '';
+    my $dec = sprintf "%${sig}f", $n;
+    $dec =~ s/\.[0]+$//g;
+    return $dec;
+  } elsif ($n =~ /^\s*([-+]?)(\d+)[\.,]*(\d*)\s*$/) {
+    return $1.$2.".".$3;
+  } elsif ($n =~ /^\s*(.*?)\s*$/) {
+    return $1;
+  } else {
+    return $n;
+  }
+}
+
+sub compatibility_methods {
+  my ($self) = @_;
+  # add_perfdata
+  # add_message
+  # nagios_exit
+  # ->{warningrange}
+  # ->{criticalrange}
+  # ...
+  $self->{warningrange} = ($self->get_thresholds())[0];
+  $self->{criticalrange} = ($self->get_thresholds())[1];
+  my $old_init = $self->can('init');
+  my %params = (
+    'mode' => join('::', split(/-/, $self->opts->mode)),
+    'name' => $self->opts->name,
+    'name2' => $self->opts->name2,
+  );
+  {
+    no strict 'refs';
+    no warnings 'redefine';
+    *{ref($self).'::init'} = sub {
+      $self->$old_init(%params);
+      $self->nagios(%params);
+    };
+    *{ref($self).'::add_nagios'} = \&{"Monitoring::GLPlugin::add_message"};
+    *{ref($self).'::add_nagios_ok'} = \&{"Monitoring::GLPlugin::add_ok"};
+    *{ref($self).'::add_nagios_warning'} = \&{"Monitoring::GLPlugin::add_warning"};
+    *{ref($self).'::add_nagios_critical'} = \&{"Monitoring::GLPlugin::add_critical"};
+    *{ref($self).'::add_nagios_unknown'} = \&{"Monitoring::GLPlugin::add_unknown"};
+    *{ref($self).'::add_perfdata'} = sub {
+      my $self = shift;
+      my $message = shift;
+      foreach my $perfdata (split(/\s+/, $message)) {
+      my ($label, $perfstr) = split(/=/, $perfdata);
+      my ($value, $warn, $crit, $min, $max) = split(/;/, $perfstr);
+      $value =~ /^([\d\.\-\+]+)(.*)$/;
+      $value = $1;
+      my $uom = $2;
+      $Monitoring::GLPlugin::plugin->add_perfdata(
+        label => $label,
+        value => $value,
+        uom => $uom,
+        warn => $warn,
+        crit => $crit,
+        min => $min,
+        max => $max,
+      );
+      }
+    };
+    *{ref($self).'::check_thresholds'} = sub {
+      my $self = shift;
+      my $value = shift;
+      my $defaultwarningrange = shift;
+      my $defaultcriticalrange = shift;
+      $Monitoring::GLPlugin::plugin->set_thresholds(
+          metric => 'default',
+          warning => $defaultwarningrange,
+          critical => $defaultcriticalrange,
+      );
+      $self->{warningrange} = ($self->get_thresholds())[0];
+      $self->{criticalrange} = ($self->get_thresholds())[1];
+      return $Monitoring::GLPlugin::plugin->check_thresholds(
+          metric => 'default',
+          value => $value,
+          warning => $defaultwarningrange,
+          critical => $defaultcriticalrange,
+      );
+    };
+  }
+}
+
+
 sub AUTOLOAD {
-  my $self = shift;
+  my ($self, @params) = @_;
   return if ($AUTOLOAD =~ /DESTROY/);
   $self->debug("AUTOLOAD %s\n", $AUTOLOAD)
         if $self->opts->verbose >= 2;
@@ -1355,7 +1477,6 @@ sub AUTOLOAD {
     my $subsystem = $2;
     my $analyze = sprintf "analyze_%s_subsystem", $subsystem;
     my $check = sprintf "check_%s_subsystem", $subsystem;
-    my @params = @_;
     if (@params) {
       # analyzer class
       my $subsystem_class = shift @params;
@@ -1374,11 +1495,11 @@ sub AUTOLOAD {
     $self->{components}->{$subsystem}->dump()
         if $self->opts->verbose >= 2;
   } elsif ($AUTOLOAD =~ /^.*::(status_code|check_messages|nagios_exit|html_string|perfdata_string|selected_perfdata|check_thresholds|get_thresholds|opts)$/) {
-    return $Monitoring::GLPlugin::plugin->$1(@_);
+    return $Monitoring::GLPlugin::plugin->$1(@params);
   } elsif ($AUTOLOAD =~ /^.*::(reduce_messages|reduce_messages_short|clear_messages|suppress_messages|add_html|add_perfdata|override_opt|create_opt|set_thresholds|force_thresholds)$/) {
-    $Monitoring::GLPlugin::plugin->$1(@_);
+    $Monitoring::GLPlugin::plugin->$1(@params);
   } elsif ($AUTOLOAD =~ /^.*::mod_arg_(.*)$/) {
-    return $Monitoring::GLPlugin::plugin->mod_arg($1, @_);
+    return $Monitoring::GLPlugin::plugin->mod_arg($1, @params);
   } else {
     $self->debug("AUTOLOAD: class %s has no method %s\n",
         ref($self), $AUTOLOAD);
